@@ -42,7 +42,34 @@ public:
 	 */
 	BstTable() { }
 
-	BstTable(long allocLen);
+	/**
+	 * \brief Construct with initial value.
+	 *
+	 * Constructs a binary search table with an initial item. Uses the default
+	 * constructor for initializing Value.
+	 */
+	BstTable(const Key &key) 
+		{ insert(key); }
+
+#if defined( BSTMAP )
+	/**
+	 * \brief Construct with initial value.
+	 *
+	 * Constructs a binary search table with an initial key/value pair.
+	 */
+	BstTable(const Key &key, const Value &val) 
+		{ insert(key, val); }
+#endif
+
+#if ! defined( BSTSET )
+	/**
+	 * \brief Construct with initial value.
+	 *
+	 * Constructs a binary search table with an initial Element.
+	 */
+	BstTable(const Element &el) 
+		{ insert(el); }
+#endif
 
 	Element *insert(const Key &key, Element **lastFound = 0);
 	Element *insertMulti(const Key &key);
@@ -189,6 +216,7 @@ public:
 };
 
 
+#if 0
 #if defined( SHARED_BST )
 /**
  * \brief Construct a binary search table with an initial amount of
@@ -235,6 +263,7 @@ template <BST_TEMPL_DEF> BstTable<BST_TEMPL_USE>::
 	}
 }
 
+#endif
 #endif
 
 /**
