@@ -26,6 +26,7 @@
 #include <iostream>
 #include <fstream>
 #include <string>
+#include "vector.h"
 #include "config.h"
 
 #define PROGNAME "ragel"
@@ -50,18 +51,29 @@ extern MinimizeLevel minimizeLevel;
 extern MinimizeOpt minimizeOpt;
 extern char *machineSpec, *machineName;
 extern bool printStatistics;
+extern bool wantDupsRemoved;
 
+/* Error reporting format. */
+enum ErrorFormat {
+	ErrorFormatGNU,
+	ErrorFormatMSVC,
+};
+
+extern ErrorFormat errorFormat;
 extern int gblErrorCount;
 extern char mainMachine[];
 
 /* Location in an input file. */
 struct InputLoc
 {
-	char *fileName;
+	const char *fileName;
 	int line;
 	int col;
 };
 
+InputLoc makeInputLoc( const char *fileName, int line = 0, int col = 0 );
+std::ostream &operator<<( std::ostream &out, const InputLoc &loc );
+
 /* Error reporting. */
 std::ostream &error();
 std::ostream &error( const InputLoc &loc ); 
@@ -71,4 +83,7 @@ void terminateAllParsers( );
 void writeMachines( std::ostream &out, std::string hostData, char *inputFileName );
 void xmlEscapeHost( std::ostream &out, char *data, long len );
 
+typedef Vector<const char *> ArgsVector;
+extern ArgsVector includePaths;
+
 #endif /* _RAGEL_H */
