@@ -44,14 +44,18 @@ CodeGenData::CodeGenData( ostream &out )
 	errState(-1),
 	getKeyExpr(0),
 	accessExpr(0),
+	prePushExpr(0),
+	postPopExpr(0),
 	pExpr(0),
 	peExpr(0),
+	eofExpr(0),
 	csExpr(0),
 	topExpr(0),
 	stackExpr(0),
 	actExpr(0),
 	tokstartExpr(0),
 	tokendExpr(0),
+	dataExpr(0),
 	wantComplete(0),
 	hasLongestMatch(false),
 	hasEnd(true),
@@ -232,7 +236,7 @@ void CodeGenData::setFinal( int snum )
 
 
 void CodeGenData::setStateActions( int snum, long toStateAction, 
-			long fromStateAction, long eofAction )
+		long fromStateAction, long eofAction )
 {
 	RedStateAp *curState = allStates + snum;
 	if ( toStateAction >= 0 )
@@ -243,6 +247,14 @@ void CodeGenData::setStateActions( int snum, long toStateAction,
 		curState->eofAction = allActionTables + eofAction;
 }
 
+void CodeGenData::setEofTrans( int snum, long eofTarget, long actId )
+{
+	RedStateAp *curState = allStates + snum;
+	RedStateAp *targState = allStates + eofTarget;
+	RedAction *eofAct = allActionTables + actId;
+	curState->eofTrans = redFsm->allocateTrans( targState, eofAct );
+}
+
 void CodeGenData::resolveTargetStates( InlineList *inlineList )
 {
 	for ( InlineList::Iter item = *inlineList; item.lte(); item++ ) {
@@ -395,6 +407,13 @@ void CodeGenData::findFinalActionRefs()
 				item->value->numTransRefs += 1;
 		}
 
+		/* Reference count eof transitions. */
+		if ( st->eofTrans != 0 && st->eofTrans->action != 0 ) {
+			st->eofTrans->action->numTransRefs += 1;
+			for ( ActionTable::Iter item = st->eofTrans->action->key; item.lte(); item++ )
+				item->value->numTransRefs += 1;
+		}
+
 		/* Reference count to state actions. */
 		if ( st->toStateAction != 0 ) {
 			st->toStateAction->numToStateRefs += 1;
@@ -535,7 +554,7 @@ void CodeGenData::setValueLimits()
 		if ( ! st.last() ) {
 			redFsm->maxCondOffset += st->stateCondList.length();
 			redFsm->maxKeyOffset += st->outSingle.length() + st->outRange.length()*2;
-			redFsm->maxIndexOffset += st->outSingle.length() + st->outRange.length() + 1;
+			redFsm->maxIndexOffset += st->outSingle.length() + st->outRange.length() + 2;
 		}
 
 		/* Max cond span. */
@@ -637,6 +656,9 @@ void CodeGenData::analyzeMachine()
 		
 		if ( st->stateCondList.length() > 0 )
 			redFsm->bAnyConditions = true;
+
+		if ( st->eofTrans != 0 )
+			redFsm->bAnyEofTrans = true;
 	}
 
 	/* Assign ids to actions that are referenced. */
@@ -692,13 +714,6 @@ void CodeGenData::writeStatement( InputLoc &loc, int nargs, char **args )
 		}
 		writeExec();
 	}
-	else if ( strcmp( args[0], "eof" ) == 0 ) {
-		for ( int i = 1; i < nargs; i++ ) {
-			source_warning(loc) << "unrecognized write option \"" << 
-					args[i] << "\"" << endl;
-		}
-		writeEOF();
-	}
 	else if ( strcmp( args[0], "exports" ) == 0 ) {
 		for ( int i = 1; i < nargs; i++ ) {
 			source_warning(loc) << "unrecognized write option \"" << 
