@@ -140,7 +140,7 @@ string FsmCodeGen::ACCESS()
 {
 	ostringstream ret;
 	if ( accessExpr != 0 )
-		INLINE_LIST( ret, accessExpr, 0, false );
+		INLINE_LIST( ret, accessExpr, 0, false, false );
 	return ret.str();
 }
 
@@ -152,7 +152,7 @@ string FsmCodeGen::P()
 		ret << "p";
 	else {
 		ret << "(";
-		INLINE_LIST( ret, pExpr, 0, false );
+		INLINE_LIST( ret, pExpr, 0, false, false );
 		ret << ")";
 	}
 	return ret.str();
@@ -165,7 +165,20 @@ string FsmCodeGen::PE()
 		ret << "pe";
 	else {
 		ret << "(";
-		INLINE_LIST( ret, peExpr, 0, false );
+		INLINE_LIST( ret, peExpr, 0, false, false );
+		ret << ")";
+	}
+	return ret.str();
+}
+
+string FsmCodeGen::EOFV()
+{
+	ostringstream ret;
+	if ( eofExpr == 0 )
+		ret << "eof";
+	else {
+		ret << "(";
+		INLINE_LIST( ret, eofExpr, 0, false, false );
 		ret << ")";
 	}
 	return ret.str();
@@ -179,7 +192,7 @@ string FsmCodeGen::CS()
 	else {
 		/* Emit the user supplied method of retrieving the key. */
 		ret << "(";
-		INLINE_LIST( ret, csExpr, 0, false );
+		INLINE_LIST( ret, csExpr, 0, false, false );
 		ret << ")";
 	}
 	return ret.str();
@@ -192,7 +205,7 @@ string FsmCodeGen::TOP()
 		ret << ACCESS() + "top";
 	else {
 		ret << "(";
-		INLINE_LIST( ret, topExpr, 0, false );
+		INLINE_LIST( ret, topExpr, 0, false, false );
 		ret << ")";
 	}
 	return ret.str();
@@ -205,7 +218,7 @@ string FsmCodeGen::STACK()
 		ret << ACCESS() + "stack";
 	else {
 		ret << "(";
-		INLINE_LIST( ret, stackExpr, 0, false );
+		INLINE_LIST( ret, stackExpr, 0, false, false );
 		ret << ")";
 	}
 	return ret.str();
@@ -218,7 +231,7 @@ string FsmCodeGen::ACT()
 		ret << ACCESS() + "act";
 	else {
 		ret << "(";
-		INLINE_LIST( ret, actExpr, 0, false );
+		INLINE_LIST( ret, actExpr, 0, false, false );
 		ret << ")";
 	}
 	return ret.str();
@@ -228,10 +241,10 @@ string FsmCodeGen::TOKSTART()
 {
 	ostringstream ret;
 	if ( tokstartExpr == 0 )
-		ret << ACCESS() + "tokstart";
+		ret << ACCESS() + "ts";
 	else {
 		ret << "(";
-		INLINE_LIST( ret, tokstartExpr, 0, false );
+		INLINE_LIST( ret, tokstartExpr, 0, false, false );
 		ret << ")";
 	}
 	return ret.str();
@@ -241,10 +254,10 @@ string FsmCodeGen::TOKEND()
 {
 	ostringstream ret;
 	if ( tokendExpr == 0 )
-		ret << ACCESS() + "tokend";
+		ret << ACCESS() + "te";
 	else {
 		ret << "(";
-		INLINE_LIST( ret, tokendExpr, 0, false );
+		INLINE_LIST( ret, tokendExpr, 0, false, false );
 		ret << ")";
 	}
 	return ret.str();
@@ -272,7 +285,7 @@ string FsmCodeGen::GET_KEY()
 	if ( getKeyExpr != 0 ) { 
 		/* Emit the user supplied method of retrieving the key. */
 		ret << "(";
-		INLINE_LIST( ret, getKeyExpr, 0, false );
+		INLINE_LIST( ret, getKeyExpr, 0, false, false );
 		ret << ")";
 	}
 	else {
@@ -310,30 +323,38 @@ void FsmCodeGen::EXEC( ostream &ret, InlineItem *item, int targState, int inFini
 	 * code. If the inline list is a single word it will get interpreted as a
 	 * C-style cast by the D compiler. */
 	ret << "{" << P() << " = ((";
-	INLINE_LIST( ret, item->children, targState, inFinish );
+	INLINE_LIST( ret, item->children, targState, inFinish, false );
 	ret << "))-1;}";
 }
 
 void FsmCodeGen::LM_SWITCH( ostream &ret, InlineItem *item, 
-		int targState, int inFinish )
+		int targState, int inFinish, bool csForced )
 {
 	ret << 
 		"	switch( " << ACT() << " ) {\n";
 
+	bool haveDefault = false;
 	for ( InlineList::Iter lma = *item->children; lma.lte(); lma++ ) {
 		/* Write the case label, the action and the case break. */
-		ret << "	case " << lma->lmId << ":\n";
+		if ( lma->lmId < 0 ) {
+			ret << "	default:\n";
+			haveDefault = true;
+		}
+		else
+			ret << "	case " << lma->lmId << ":\n";
 
 		/* Write the block and close it off. */
 		ret << "	{";
-		INLINE_LIST( ret, lma->children, targState, inFinish );
+		INLINE_LIST( ret, lma->children, targState, inFinish, csForced );
 		ret << "}\n";
 
 		ret << "	break;\n";
 	}
-	/* Default required for D code. */
+
+	if ( hostLang->lang == HostLang::D && !haveDefault )
+		ret << "	default: break;";
+
 	ret << 
-		"	default: break;\n"
 		"	}\n"
 		"\t";
 }
@@ -373,12 +394,12 @@ void FsmCodeGen::SET_TOKSTART( ostream &ret, InlineItem *item )
 }
 
 void FsmCodeGen::SUB_ACTION( ostream &ret, InlineItem *item, 
-		int targState, bool inFinish )
+		int targState, bool inFinish, bool csForced )
 {
 	if ( item->children->length() > 0 ) {
 		/* Write the block and close it off. */
 		ret << "{";
-		INLINE_LIST( ret, item->children, targState, inFinish );
+		INLINE_LIST( ret, item->children, targState, inFinish, csForced );
 		ret << "}";
 	}
 }
@@ -387,7 +408,7 @@ void FsmCodeGen::SUB_ACTION( ostream &ret, InlineItem *item,
 /* Write out an inline tree structure. Walks the list and possibly calls out
  * to virtual functions than handle language specific items in the tree. */
 void FsmCodeGen::INLINE_LIST( ostream &ret, InlineList *inlineList, 
-		int targState, bool inFinish )
+		int targState, bool inFinish, bool csForced )
 {
 	for ( InlineList::Iter item = *inlineList; item.lte(); item++ ) {
 		switch ( item->type ) {
@@ -437,7 +458,7 @@ void FsmCodeGen::INLINE_LIST( ostream &ret, InlineList *inlineList,
 			NEXT_EXPR( ret, item, inFinish );
 			break;
 		case InlineItem::LmSwitch:
-			LM_SWITCH( ret, item, targState, inFinish );
+			LM_SWITCH( ret, item, targState, inFinish, csForced );
 			break;
 		case InlineItem::LmSetActId:
 			SET_ACT( ret, item );
@@ -458,10 +479,10 @@ void FsmCodeGen::INLINE_LIST( ostream &ret, InlineList *inlineList,
 			SET_TOKSTART( ret, item );
 			break;
 		case InlineItem::SubAction:
-			SUB_ACTION( ret, item, targState, inFinish );
+			SUB_ACTION( ret, item, targState, inFinish, csForced );
 			break;
 		case InlineItem::Break:
-			BREAK( ret, targState );
+			BREAK( ret, targState, csForced );
 			break;
 		}
 	}
@@ -479,14 +500,15 @@ string FsmCodeGen::LDIR_PATH( char *path )
 	return ret.str();
 }
 
-void FsmCodeGen::ACTION( ostream &ret, Action *action, int targState, bool inFinish )
+void FsmCodeGen::ACTION( ostream &ret, Action *action, int targState, 
+		bool inFinish, bool csForced )
 {
 	/* Write the preprocessor line info for going into the source file. */
 	lineDirective( ret, sourceFileName, action->loc.line );
 
 	/* Write the block and close it off. */
 	ret << "\t{";
-	INLINE_LIST( ret, action->inlineList, targState, inFinish );
+	INLINE_LIST( ret, action->inlineList, targState, inFinish, csForced );
 	ret << "}\n";
 }
 
@@ -494,7 +516,7 @@ void FsmCodeGen::CONDITION( ostream &ret, Action *condition )
 {
 	ret << "\n";
 	lineDirective( ret, sourceFileName, condition->loc.line );
-	INLINE_LIST( ret, condition->inlineList, 0, false );
+	INLINE_LIST( ret, condition->inlineList, 0, false, false );
 }
 
 string FsmCodeGen::ERROR_STATE()
