@@ -42,7 +42,7 @@ std::ostream &GotoCodeGen::TO_STATE_ACTION_SWITCH()
 		if ( act->numToStateRefs > 0 ) {
 			/* Write the case label, the action and the case break. */
 			out << "\tcase " << act->actionId << ":\n";
-			ACTION( out, act, 0, false );
+			ACTION( out, act, 0, false, false );
 			out << "\tbreak;\n";
 		}
 	}
@@ -59,7 +59,7 @@ std::ostream &GotoCodeGen::FROM_STATE_ACTION_SWITCH()
 		if ( act->numFromStateRefs > 0 ) {
 			/* Write the case label, the action and the case break. */
 			out << "\tcase " << act->actionId << ":\n";
-			ACTION( out, act, 0, false );
+			ACTION( out, act, 0, false, false );
 			out << "\tbreak;\n";
 		}
 	}
@@ -76,7 +76,7 @@ std::ostream &GotoCodeGen::EOF_ACTION_SWITCH()
 		if ( act->numEofRefs > 0 ) {
 			/* Write the case label, the action and the case break. */
 			out << "\tcase " << act->actionId << ":\n";
-			ACTION( out, act, 0, true );
+			ACTION( out, act, 0, true, false );
 			out << "\tbreak;\n";
 		}
 	}
@@ -93,7 +93,7 @@ std::ostream &GotoCodeGen::ACTION_SWITCH()
 		if ( act->numTransRefs > 0 ) {
 			/* Write the case label, the action and the case break. */
 			out << "\tcase " << act->actionId << ":\n";
-			ACTION( out, act, 0, false );
+			ACTION( out, act, 0, false, false );
 			out << "\tbreak;\n";
 		}
 	}
@@ -550,7 +550,7 @@ void GotoCodeGen::GOTO( ostream &ret, int gotoDest, bool inFinish )
 void GotoCodeGen::GOTO_EXPR( ostream &ret, InlineItem *ilItem, bool inFinish )
 {
 	ret << "{" << CS() << " = (";
-	INLINE_LIST( ret, ilItem->children, 0, inFinish );
+	INLINE_LIST( ret, ilItem->children, 0, inFinish, false );
 	ret << "); " << CTRL_FLOW() << "goto _again;}";
 }
 
@@ -572,33 +572,56 @@ void GotoCodeGen::NEXT( ostream &ret, int nextDest, bool inFinish )
 void GotoCodeGen::NEXT_EXPR( ostream &ret, InlineItem *ilItem, bool inFinish )
 {
 	ret << CS() << " = (";
-	INLINE_LIST( ret, ilItem->children, 0, inFinish );
+	INLINE_LIST( ret, ilItem->children, 0, inFinish, false );
 	ret << ");";
 }
 
 void GotoCodeGen::CALL( ostream &ret, int callDest, int targState, bool inFinish )
 {
+	if ( prePushExpr != 0 ) {
+		ret << "{";
+		INLINE_LIST( ret, prePushExpr, 0, false, false );
+	}
+
 	ret << "{" << STACK() << "[" << TOP() << "++] = " << CS() << "; " << CS() << " = " << 
 			callDest << "; " << CTRL_FLOW() << "goto _again;}";
+
+	if ( prePushExpr != 0 )
+		ret << "}";
 }
 
 void GotoCodeGen::CALL_EXPR( ostream &ret, InlineItem *ilItem, int targState, bool inFinish )
 {
+	if ( prePushExpr != 0 ) {
+		ret << "{";
+		INLINE_LIST( ret, prePushExpr, 0, false, false );
+	}
+
 	ret << "{" << STACK() << "[" << TOP() << "++] = " << CS() << "; " << CS() << " = (";
-	INLINE_LIST( ret, ilItem->children, targState, inFinish );
+	INLINE_LIST( ret, ilItem->children, targState, inFinish, false );
 	ret << "); " << CTRL_FLOW() << "goto _again;}";
+
+	if ( prePushExpr != 0 )
+		ret << "}";
 }
 
 void GotoCodeGen::RET( ostream &ret, bool inFinish )
 {
-	ret << "{" << CS() << " = " << STACK() << "[--" << TOP() << "]; " << 
-			CTRL_FLOW() << "goto _again;}";
+	ret << "{" << CS() << " = " << STACK() << "[--" << TOP() << "];";
+
+	if ( postPopExpr != 0 ) {
+		ret << "{";
+		INLINE_LIST( ret, postPopExpr, 0, false, false );
+		ret << "}";
+	}
+
+	ret << CTRL_FLOW() << "goto _again;}";
 }
 
-void GotoCodeGen::BREAK( ostream &ret, int targState )
+void GotoCodeGen::BREAK( ostream &ret, int targState, bool csForced )
 {
 	outLabelUsed = true;
-	ret << CTRL_FLOW() << "goto _out;";
+	ret << "{" << P() << "++; " << CTRL_FLOW() << "goto _out; }";
 }
 
 void GotoCodeGen::writeData()
@@ -636,6 +659,7 @@ void GotoCodeGen::writeData()
 
 void GotoCodeGen::writeExec()
 {
+	testEofUsed = false;
 	outLabelUsed = false;
 
 	out << "	{\n";
@@ -657,10 +681,10 @@ void GotoCodeGen::writeExec()
 	out << "\n";
 
 	if ( hasEnd ) {
-		outLabelUsed = true;
+		testEofUsed = true;
 		out << 
 			"	if ( " << P() << " == " << PE() << " )\n"
-			"		goto _out;\n";
+			"		goto _test_eof;\n";
 	}
 
 	if ( redFsm->errState != 0 ) {
@@ -730,27 +754,48 @@ void GotoCodeGen::writeExec()
 			"	goto _resume;\n";
 	}
 
+	if ( testEofUsed )
+		out << "	_test_eof: {}\n";
+
+	if ( redFsm->anyEofTrans() || redFsm->anyEofActions() ) {
+		out << 
+			"	if ( " << P() << " == " << EOFV() << " )\n"
+			"	{\n";
+
+		if ( redFsm->anyEofTrans() ) {
+			out <<
+				"	switch ( " << CS() << " ) {\n";
+
+			for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
+				if ( st->eofTrans != 0 )
+					out << "	case " << st->id << ": goto tr" << st->eofTrans->id << ";\n";
+			}
+
+			SWITCH_DEFAULT() <<
+				"	}\n";
+		}
+
+		if ( redFsm->anyEofActions() ) {
+			out <<
+				"	" << PTR_CONST() << ARRAY_TYPE(redFsm->maxActArrItem) << 
+						POINTER() << "__acts = " << 
+						ARR_OFF( A(), EA() + "[" + CS() + "]" ) << ";\n"
+				"	" << UINT() << " __nacts = " << CAST(UINT()) << " *__acts++;\n"
+				"	while ( __nacts-- > 0 ) {\n"
+				"		switch ( *__acts++ ) {\n";
+				EOF_ACTION_SWITCH();
+				SWITCH_DEFAULT() <<
+				"		}\n"
+				"	}\n";
+		}
+
+		out <<
+			"	}\n"
+			"\n";
+	}
+
 	if ( outLabelUsed )
 		out << "	_out: {}\n";
 
 	out << "	}\n";
 }
-
-void GotoCodeGen::writeEOF()
-{
-	if ( redFsm->anyEofActions() ) {
-		out << 
-			"	{\n"
-			"	" << PTR_CONST() << ARRAY_TYPE(redFsm->maxActArrItem) << POINTER() << "_acts = " << 
-					ARR_OFF( A(), EA() + "[" + CS() + "]" ) << ";\n"
-			"	" << UINT() << " _nacts = " << CAST(UINT()) << " *_acts++;\n"
-			"	while ( _nacts-- > 0 ) {\n"
-			"		switch ( *_acts++ ) {\n";
-			EOF_ACTION_SWITCH();
-			SWITCH_DEFAULT() <<
-			"		}\n"
-			"	}\n"
-			"	}\n"
-			"\n";
-	}
-}
