@@ -100,7 +100,7 @@ std::ostream &TabCodeGen::TO_STATE_ACTION_SWITCH()
 		if ( act->numToStateRefs > 0 ) {
 			/* Write the case label, the action and the case break. */
 			out << "\tcase " << act->actionId << ":\n";
-			ACTION( out, act, 0, false );
+			ACTION( out, act, 0, false, false );
 			out << "\tbreak;\n";
 		}
 	}
@@ -117,7 +117,7 @@ std::ostream &TabCodeGen::FROM_STATE_ACTION_SWITCH()
 		if ( act->numFromStateRefs > 0 ) {
 			/* Write the case label, the action and the case break. */
 			out << "\tcase " << act->actionId << ":\n";
-			ACTION( out, act, 0, false );
+			ACTION( out, act, 0, false, false );
 			out << "\tbreak;\n";
 		}
 	}
@@ -134,7 +134,7 @@ std::ostream &TabCodeGen::EOF_ACTION_SWITCH()
 		if ( act->numEofRefs > 0 ) {
 			/* Write the case label, the action and the case break. */
 			out << "\tcase " << act->actionId << ":\n";
-			ACTION( out, act, 0, true );
+			ACTION( out, act, 0, true, false );
 			out << "\tbreak;\n";
 		}
 	}
@@ -152,7 +152,7 @@ std::ostream &TabCodeGen::ACTION_SWITCH()
 		if ( act->numTransRefs > 0 ) {
 			/* Write the case label, the action and the case break. */
 			out << "\tcase " << act->actionId << ":\n";
-			ACTION( out, act, 0, false );
+			ACTION( out, act, 0, false, false );
 			out << "\tbreak;\n";
 		}
 	}
@@ -327,6 +327,30 @@ std::ostream &TabCodeGen::EOF_ACTIONS()
 	return out;
 }
 
+std::ostream &TabCodeGen::EOF_TRANS()
+{
+	out << "\t";
+	int totalStateNum = 0;
+	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
+		/* Write any eof action. */
+		long trans = 0;
+		if ( st->eofTrans != 0 ) {
+			assert( st->eofTrans->pos >= 0 );
+			trans = st->eofTrans->pos+1;
+		}
+		out << trans;
+
+		if ( !st.last() ) {
+			out << ", ";
+			if ( ++totalStateNum % IALL == 0 )
+				out << "\n\t";
+		}
+	}
+	out << "\n";
+	return out;
+}
+
+
 std::ostream &TabCodeGen::COND_KEYS()
 {
 	out << '\t';
@@ -467,6 +491,18 @@ std::ostream &TabCodeGen::TRANS_TARGS()
 		}
 	}
 
+	/* Add any eof transitions that have not yet been written out above. */
+	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
+		if ( st->eofTrans != 0 ) {
+			RedTransAp *trans = st->eofTrans;
+			trans->pos = totalTrans;
+			out << trans->targ->id << ", ";
+			if ( ++totalTrans % IALL == 0 )
+				out << "\n\t";
+		}
+	}
+
+
 	/* Output one last number so we don't have to figure out when the last
 	 * entry is and avoid writing a comma. */
 	out << 0 << "\n";
@@ -504,6 +540,17 @@ std::ostream &TabCodeGen::TRANS_ACTIONS()
 		}
 	}
 
+	/* Add any eof transitions that have not yet been written out above. */
+	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
+		if ( st->eofTrans != 0 ) {
+			RedTransAp *trans = st->eofTrans;
+			TRANS_ACTION( trans ) << ", ";
+			if ( ++totalTrans % IALL == 0 )
+				out << "\n\t";
+		}
+	}
+
+
 	/* Output one last number so we don't have to figure out when the last
 	 * entry is and avoid writing a comma. */
 	out << 0 << "\n";
@@ -521,8 +568,11 @@ std::ostream &TabCodeGen::TRANS_TARGS_WI()
 	out << '\t';
 	int totalStates = 0;
 	for ( int t = 0; t < redFsm->transSet.length(); t++ ) {
-		/* Write out the target state. */
+		/* Record the position, need this for eofTrans. */
 		RedTransAp *trans = transPtrs[t];
+		trans->pos = t;
+
+		/* Write out the target state. */
 		out << trans->targ->id;
 		if ( t < redFsm->transSet.length()-1 ) {
 			out << ", ";
@@ -623,7 +673,7 @@ void TabCodeGen::GOTO( ostream &ret, int gotoDest, bool inFinish )
 void TabCodeGen::GOTO_EXPR( ostream &ret, InlineItem *ilItem, bool inFinish )
 {
 	ret << "{" << CS() << " = (";
-	INLINE_LIST( ret, ilItem->children, 0, inFinish );
+	INLINE_LIST( ret, ilItem->children, 0, inFinish, false );
 	ret << "); " << CTRL_FLOW() << "goto _again;}";
 }
 
@@ -645,33 +695,57 @@ void TabCodeGen::NEXT( ostream &ret, int nextDest, bool inFinish )
 void TabCodeGen::NEXT_EXPR( ostream &ret, InlineItem *ilItem, bool inFinish )
 {
 	ret << CS() << " = (";
-	INLINE_LIST( ret, ilItem->children, 0, inFinish );
+	INLINE_LIST( ret, ilItem->children, 0, inFinish, false );
 	ret << ");";
 }
 
 void TabCodeGen::CALL( ostream &ret, int callDest, int targState, bool inFinish )
 {
+	if ( prePushExpr != 0 ) {
+		ret << "{";
+		INLINE_LIST( ret, prePushExpr, 0, false, false );
+	}
+
 	ret << "{" << STACK() << "[" << TOP() << "++] = " << CS() << "; " << CS() << " = " << 
 			callDest << "; " << CTRL_FLOW() << "goto _again;}";
+
+	if ( prePushExpr != 0 )
+		ret << "}";
 }
 
 void TabCodeGen::CALL_EXPR( ostream &ret, InlineItem *ilItem, int targState, bool inFinish )
 {
+	if ( prePushExpr != 0 ) {
+		ret << "{";
+		INLINE_LIST( ret, prePushExpr, 0, false, false );
+	}
+
 	ret << "{" << STACK() << "[" << TOP() << "++] = " << CS() << "; " << CS() << " = (";
-	INLINE_LIST( ret, ilItem->children, targState, inFinish );
+	INLINE_LIST( ret, ilItem->children, targState, inFinish, false );
 	ret << "); " << CTRL_FLOW() << "goto _again;}";
+
+	if ( prePushExpr != 0 )
+		ret << "}";
 }
 
 void TabCodeGen::RET( ostream &ret, bool inFinish )
 {
 	ret << "{" << CS() << " = " << STACK() << "[--" << 
-			TOP() << "]; " << CTRL_FLOW() <<  "goto _again;}";
+			TOP() << "]; ";
+
+	if ( postPopExpr != 0 ) {
+		ret << "{";
+		INLINE_LIST( ret, postPopExpr, 0, false, false );
+		ret << "}";
+	}
+
+	ret << CTRL_FLOW() <<  "goto _again;}";
 }
 
-void TabCodeGen::BREAK( ostream &ret, int targState )
+void TabCodeGen::BREAK( ostream &ret, int targState, bool csForced )
 {
 	outLabelUsed = true;
-	ret << CTRL_FLOW() << "goto _out;";
+	ret << "{" << P() << "++; " << CTRL_FLOW() << "goto _out; }";
 }
 
 void TabCodeGen::writeData()
@@ -785,6 +859,13 @@ void TabCodeGen::writeData()
 		"\n";
 	}
 
+	if ( redFsm->anyEofTrans() ) {
+		OPEN_ARRAY( ARRAY_TYPE(redFsm->maxIndexOffset+1), ET() );
+		EOF_TRANS();
+		CLOSE_ARRAY() <<
+		"\n";
+	}
+
 	STATE_IDS();
 }
 
@@ -843,6 +924,7 @@ void TabCodeGen::COND_TRANSLATE()
 
 void TabCodeGen::writeExec()
 {
+	testEofUsed = false;
 	outLabelUsed = false;
 
 	out <<
@@ -863,7 +945,8 @@ void TabCodeGen::writeExec()
 			|| redFsm->anyFromStateActions() )
 	{
 		out << 
-			"	" << PTR_CONST() << ARRAY_TYPE(redFsm->maxActArrItem) << POINTER() << "_acts;\n"
+			"	" << PTR_CONST() << ARRAY_TYPE(redFsm->maxActArrItem) << 
+					POINTER() << "_acts;\n"
 			"	" << UINT() << " _nacts;\n";
 	}
 
@@ -872,10 +955,10 @@ void TabCodeGen::writeExec()
 		"\n";
 
 	if ( hasEnd ) {
-		outLabelUsed = true;
+		testEofUsed = true;
 		out << 
 			"	if ( " << P() << " == " << PE() << " )\n"
-			"		goto _out;\n";
+			"		goto _test_eof;\n";
 	}
 
 	if ( redFsm->errState != 0 ) {
@@ -907,11 +990,14 @@ void TabCodeGen::writeExec()
 
 	out << "_match:\n";
 
-	if ( redFsm->anyRegCurStateRef() )
-		out << "	_ps = " << CS() << ";\n";
-
 	if ( useIndicies )
 		out << "	_trans = " << I() << "[_trans];\n";
+	
+	if ( redFsm->anyEofTrans() )
+		out << "_eof_trans:\n";
+
+	if ( redFsm->anyRegCurStateRef() )
+		out << "	_ps = " << CS() << ";\n";
 
 	out <<
 		"	" << CS() << " = " << TT() << "[_trans];\n"
@@ -968,28 +1054,43 @@ void TabCodeGen::writeExec()
 			"	goto _resume;\n";
 	}
 	
+	if ( testEofUsed )
+		out << "	_test_eof: {}\n";
+	
+	if ( redFsm->anyEofTrans() || redFsm->anyEofActions() ) {
+		out << 
+			"	if ( " << P() << " == " << EOFV() << " )\n"
+			"	{\n";
+
+		if ( redFsm->anyEofTrans() ) {
+			out <<
+				"	if ( " << ET() << "[" << CS() << "] > 0 ) {\n"
+				"		_trans = " << ET() << "[" << CS() << "] - 1;\n"
+				"		goto _eof_trans;\n"
+				"	}\n";
+		}
+
+		if ( redFsm->anyEofActions() ) {
+			out <<
+				"	" << PTR_CONST() << ARRAY_TYPE(redFsm->maxActArrItem) << 
+						POINTER() << "__acts = " << 
+						ARR_OFF( A(), EA() + "[" + CS() + "]" ) << ";\n"
+				"	" << UINT() << " __nacts = " << CAST(UINT()) << " *__acts++;\n"
+				"	while ( __nacts-- > 0 ) {\n"
+				"		switch ( *__acts++ ) {\n";
+				EOF_ACTION_SWITCH();
+				SWITCH_DEFAULT() <<
+				"		}\n"
+				"	}\n";
+		}
+		
+		out << 
+			"	}\n"
+			"\n";
+	}
+
 	if ( outLabelUsed )
 		out << "	_out: {}\n";
 
 	out << "	}\n";
 }
-
-
-void TabCodeGen::writeEOF()
-{
-	if ( redFsm->anyEofActions() ) {
-		out << 
-			"	{\n"
-			"	" << PTR_CONST() << ARRAY_TYPE(redFsm->maxActArrItem) << POINTER() << "_acts = " << 
-					ARR_OFF( A(), EA() + "[" + CS() + "]" ) << ";\n"
-			"	" << UINT() << " _nacts = " << CAST(UINT()) << " *_acts++;\n"
-			"	while ( _nacts-- > 0 ) {\n"
-			"		switch ( *_acts++ ) {\n";
-			EOF_ACTION_SWITCH();
-			SWITCH_DEFAULT() <<
-			"		}\n"
-			"	}\n"
-			"	}\n"
-			"\n";
-	}
-}
